@@ -1,10 +1,10 @@
 /*
  * scrm is an implementation of the Sequential-Coalescent-with-Recombination Model.
- * 
+ *
  * Copyright (C) 2013, 2014 Paul R. Staab, Sha (Joe) Zhu, Dirk Metzler and Gerton Lunter
- * 
+ *
  * This file is part of scrm.
- * 
+ *
  * scrm is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -44,19 +44,19 @@ class ReverseConstNodeIterator;
 class NodeContainer {
  public:
   NodeContainer();
-  ~NodeContainer() { 
-    clear(); 
+  ~NodeContainer() {
+    clear();
     for (std::vector<Node>* lane : node_lanes_) delete lane;
   };
 
   NodeContainer& operator=(NodeContainer nc) {
-    swap(*this, nc);  
+    swap(*this, nc);
     return(*this);
   };
 
   NodeContainer(const NodeContainer &nc);
-    
-  NodeIterator iterator(); 
+
+  NodeIterator iterator();
   NodeIterator iterator(Node* node);
   ConstNodeIterator iterator() const;
   ConstNodeIterator iterator(Node* node) const;
@@ -64,7 +64,7 @@ class NodeContainer {
   ReverseConstNodeIterator reverse_iterator(Node* node) const;
 
   // Create Nodes
-  Node* createNode(double height, size_t label = 0) { 
+  Node* createNode(double height, size_t label = 0) {
     // Use the slot of a previously deleted node if possible
     if (free_slots_.size() > 0) {
       Node* node = free_slots_.top();
@@ -82,9 +82,33 @@ class NodeContainer {
         new_lane->reserve(10000);
         node_lanes_.push_back(new_lane);
       }
-    } 
+    }
     (*node_lanes_.at(lane_counter_))[node_counter_] = Node(height, label);
-    return &*(node_lanes_[lane_counter_]->begin() + node_counter_++); 
+    return &*(node_lanes_[lane_counter_]->begin() + node_counter_++);
+  }
+
+  // Create Nodes
+  Node* createNode(const Node copiedNode) {
+    // Use the slot of a previously deleted node if possible
+    if (free_slots_.size() > 0) {
+      Node* node = free_slots_.top();
+      free_slots_.pop();
+      *node = Node(copiedNode);
+      return node;
+    }
+
+    // Otherwise, use a new slot
+    if (node_counter_ >= 10000) {
+      ++lane_counter_;
+      node_counter_ = 0;
+      if (lane_counter_ == node_lanes_.size()) {
+        std::vector<Node>* new_lane = new std::vector<Node>();
+        new_lane->reserve(10000);
+        node_lanes_.push_back(new_lane);
+      }
+    }
+    (*node_lanes_.at(lane_counter_))[node_counter_] = Node(copiedNode);
+    return &*(node_lanes_[lane_counter_]->begin() + node_counter_++);
   }
 
   void push_back(Node* node);
@@ -94,14 +118,14 @@ class NodeContainer {
   void move(Node *node, const double new_height);
   void clear();
 
-  Node* at(size_t nr) const; 
-  Node const* get(size_t nr) const { return at(nr); }; 
+  Node* at(size_t nr) const;
+  Node const* get(size_t nr) const { return at(nr); };
 
   Node* first() const { return first_node_; };
   Node* last() const { return last_node_; };
-  
-  size_t size() const { return size_; };  
-  bool sorted() const; 
+
+  size_t size() const { return size_; };
+  bool sorted() const;
 
 #ifdef UNITTEST
   friend class TestNodeContainer;
@@ -119,7 +143,7 @@ class NodeContainer {
 
   Node* first_node_;
   Node* last_node_;
-  
+
   void set_first(Node* node) { first_node_ = node; }
   void set_last(Node* node) { last_node_ = node; }
 
@@ -141,14 +165,14 @@ class NodeIterator {
   NodeIterator(Node* node) { current_node_ = node; };
   ~NodeIterator() {};
 
-  Node* operator*() { 
-    if (current_node_ == NULL) 
+  Node* operator*() {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    return current_node_; 
+    return current_node_;
   }
-  
+
   Node* operator++() {
-    if (current_node_ == NULL) 
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
 
     if ( current_node_->is_last() ) current_node_ = NULL;
@@ -157,25 +181,25 @@ class NodeIterator {
   }
 
   Node* operator--() {
-    if (current_node_ == NULL) 
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
 
     if ( current_node_->is_first() ) current_node_ = NULL;
     else current_node_ = current_node_->previous();
     return current_node_;
   }
-  
+
   Node* operator++(int) {
-    if (current_node_ == NULL) 
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    
+
     Node* ret = current_node_;
     if ( current_node_->is_last() ) current_node_ = NULL;
     else current_node_ = current_node_->next();
     return ret;
   }
 
-  bool good() const { 
+  bool good() const {
     return current_node_ != NULL;
   }
 
@@ -183,7 +207,7 @@ class NodeIterator {
     if ( good() ) return current_node_->height();
     else return DBL_MAX;
   }
-  
+
 #ifdef UNITTEST
   friend class TestNodeContainer;
 #endif
@@ -200,14 +224,14 @@ class ConstNodeIterator {
   ConstNodeIterator(Node const* node) { current_node_ = node; };
   ~ConstNodeIterator() {};
 
-  Node const* operator*() { 
-    if (current_node_ == NULL) 
+  Node const* operator*() {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    return current_node_; 
+    return current_node_;
   }
-  
-  Node const* operator++() { 
-    if (current_node_ == NULL) 
+
+  Node const* operator++() {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
 
     if ( current_node_->is_last() ) current_node_ = NULL;
@@ -215,27 +239,27 @@ class ConstNodeIterator {
 
     return current_node_;
   }
-  
+
   Node const* operator--() {
-    if (current_node_ == NULL) 
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
 
     if ( current_node_->is_first() ) current_node_ = NULL;
     else current_node_ = current_node_->previous();
     return current_node_;
   }
-  
-  Node const* operator++(int) { 
-    if (current_node_ == NULL) 
+
+  Node const* operator++(int) {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    
+
     Node const* ret = current_node_;
     if ( current_node_->is_last() ) current_node_ = NULL;
     else current_node_ = current_node_->next();
     return ret;
   }
- 
-  bool good() const { 
+
+  bool good() const {
     return current_node_ != NULL;
   }
 
@@ -256,14 +280,14 @@ class ReverseConstNodeIterator {
   ReverseConstNodeIterator(Node const* node) { current_node_ = node; };
   ~ReverseConstNodeIterator() {};
 
-  Node const* operator*() { 
-    if (current_node_ == NULL) 
+  Node const* operator*() {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    return current_node_; 
+    return current_node_;
   }
 
-  Node const* operator++() { 
-    if (current_node_ == NULL) 
+  Node const* operator++() {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
 
     if ( current_node_->is_first() ) current_node_ = NULL;
@@ -271,23 +295,23 @@ class ReverseConstNodeIterator {
     return current_node_;
   }
 
-  Node const* operator++(int) { 
-    if (current_node_ == NULL) 
+  Node const* operator++(int) {
+    if (current_node_ == NULL)
       throw std::out_of_range("Node iterator out of range");
-    
+
     Node const* ret = current_node_;
     if ( current_node_->is_first() ) current_node_ = NULL;
     else current_node_ = current_node_->previous();
     return ret;
   }
 
-  bool good() const { 
+  bool good() const {
     return current_node_ != NULL;
   }
 
  private:
   Node const* current_node_;
-}; 
+};
 
 
 inline NodeIterator NodeContainer::iterator() { return NodeIterator(*this); }
